# --------------------------------------------------------------------------
# PyTeddy version .9 Apr 14, 2005
# Program versions: Blender 2.36+
# --------------------------------------------------------------------------
# ***** BEGIN GPL LICENSE BLOCK *****
#
# Copyright (C) 2005: Vamsi Krishna.V vkrishna _at_ research.iiit.ac.in, vkrishna _at_students.iiit.ac.in
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------

#NOTE:
# This script implements the method suggested by Teddy: A Sketch Interface for 3D Freeform Design,
# which was published in SIGRAPH '99 by Takeo Igarishi, Satoshi Matsuoka and Hidenhiko Tanaka from
# University of Tokyo and Tokyo Institute of Technology.

import Blender
from Blender import BGL, Draw, Image
from Blender import Object, Scene, Text, Window, Registry
from Blender.BGL import *

import poly
import model
import sys
import os
import display
import makeskel

#button ids
BUTTON_QUIT = 1
BUTTON_SHOWTRIS = 2
BUTTON_DONE = 3

#button vars
bQuit= None
bDone = None
bShowTris = None

#flag variables 
fLMouse = False
fShowTris = False

#list of points clicked
pts = []

#co-ords of the canvas area
ww= 640
wh= 480
xm= 200
ym= 50

#check if the given point is in the canvas area
def inBox( x, y ):
	if x > xm and x < xm+ww :
		if( y > ym and y < ym+wh ):
			return True
	return False
	
def button_event( ev ):
	global pts
	global fShowTris
	
	if ev == BUTTON_QUIT:
		Draw.Exit( )
		Window.RedrawAll( )
	elif ev == BUTTON_SHOWTRIS:
		fShowTris = bShowTris.val	
	elif ev == BUTTON_DONE:	
		#perform some kinda optimization of the points
		approx = poly.polyConstruct( pts )
		pts = approx
		fp = open( "first.poly", "w" )
		#filling in the vertices
		fp.write( str( len( pts ) ) + " 2 0 1\n" )
		for i in xrange( len( pts ) ):
			fp.write( str(i+1) + " " + str(pts[i].x) + " " + str(pts[i].y) + " 2 \n" )
			#filling in the edges
		fp.write( str(len(pts)) + " 1\n" )
		for i in xrange( len( pts ) - 1 ):
			fp.write( str(i+1) + " " + str(i+1) + " " + str(i+2) + " 2 \n" )	
		fp.write( str(len(pts)) +  " " + str(len(pts)) + " 1 2 \n" )
		#zero holes so this
		fp.write( "0\n" )
		fp.close( )
		#triangulation of the polygon
		if fShowTris:
			os.system( "./showme first.poly" )
		os.system( "./triangle -p first" )
		if fShowTris:
			os.system( "./showme first.1" )
		#start the teddy stuff
		display.loadfile( "first.1" )
		#display.init( )
		makeskel.Run( )
		Draw.Exit()
		Window.RedrawAll( )
		
def input_event( ev, val ):
	global pts, xm, ym
	global fLMouse

	if ev == Draw.LEFTMOUSE:
		fLMouse = True

	if val == 0:
		fLMouse = False
		
	if ev == Draw.MOUSEX or ev == Draw.MOUSEY:
		#if fDraw and fLMouse:
		if fLMouse:
			x,y = Window.GetMouseCoords( )
			if( inBox( x, y ) ):
				pt = model.Point3D( )
				pt.setVal( x-xm, y-ym, 0 )
				pts.append(pt)
	Draw.Redraw(1)

def renderGui(  ):
	global pts
	global bQuit, bDone, bShowTris
	global fShowTris

	glClearColor( 130/255.0, 130/255.0, 130/255.0, 1 )
	glClear( GL_COLOR_BUFFER_BIT )

	verts = [ (xm,ym), (xm,ym+wh), (xm+ww,ym+wh), (xm+ww,ym) ]
	glColor3f( 0, 0, 0 )
	glBegin( GL_QUADS )
	map( lambda x: glVertex2i( x[0], x[1] ), verts )
	glEnd( )
		
	glColor3f( 1, 1, 1 )
	glPointSize( 5 )
	glBegin( GL_LINE_LOOP )
	for pt in pts:
		glVertex2f( pt.x+xm, pt.y+ym )
	glEnd( )
	bQuit = Draw.PushButton('Quit', BUTTON_QUIT, 5, 5, 100, 20, 'Exits the script' )
	bDone = Draw.PushButton('Done', BUTTON_DONE, 5, 50, 100, 20, 'Creates Model' )
	bShowTris = Draw.Toggle('ShowTris', BUTTON_SHOWTRIS, 5, 100, 100, 20, fShowTris, 'Shows Triangulation of Sketch' )
	glRasterPos2i( 150, 20 )
	Draw.Text( 'PyTeddy at http://research.iiit.ac.in/~vkrishna/pyteddy.html' )
	
#Register the gui function and input handlers
Draw.Register( renderGui, input_event, button_event )
