# --------------------------------------------------------------------------
# PyTeddy version .9 Apr 14, 2005
# Program versions: Blender 2.36+
# --------------------------------------------------------------------------
# ***** BEGIN GPL LICENSE BLOCK *****
#
# Copyright (C) 2005: Vamsi Krishna.V vkrishna _at_ research.iiit.ac.in, vkrishna _at_students.iiit.ac.in
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------

import Blender
from Blender import Armature as A
from Blender.Mathutils import *

import sys
import re
import model
import cal3d
import poly
import display

lines = []
range = 1
skel = cal3d.Bone( )

def init():
	global lines	
	global skel
	fp = open( "points.txt", "r" )
	txtlines = fp.readlines( )
	fp.close( )
	for line in txtlines:
		tokens = line.split(' ')	
		p1 = model.Point3D( )
		p1.setVal( float(tokens[0]), float(tokens[1]), 0 )
		p2 = model.Point3D( )
		p2.setVal( float(tokens[2]), float(tokens[3]), 0 )
		tmpLine = model.Line( )
		tmpLine.setPts( p1, p2 )
		lines.append( tmpLine )
	skel = cal3d.genCal3DSkel( lines )
	skel.optimizeBone( )
		
#function to make bones for blender
def fb_bone( bone, armatureData ):						
	if ( bone.deleted ):
		print "FOUND DELETED BONE"
		return fb_bone( bone.children[0], armatureData )		
	else:
		eb = A.Editbone( )
		eb.tail = Vector( bone.pt.x, bone.pt.y, bone.pt.z )
		ebname = 'bone'+str(bone.id).zfill(3)
		armatureData.bones[ebname] = eb

	for child in bone.children:
		bone = fb_bone( child, armatureData )	
		bone.parent = eb 
		bone.options = [ A.HINGE, A.CONNECTED ]
	return eb

def flipFace(v):
	if len(v) == 3: v[0], v[1], v[2] = v[2], v[1], v[0]
	elif len(v) == 4: v[0], v[1], v[2], v[3] = v[3], v[2], v[1], v[0]
				
def addMesh( ):
	verts = display.points
	faces = display.triangles
	mesh = Blender.NMesh.New()
	mesh.name = "TeddyMesh"
	for v in verts:
		bvert = Blender.NMesh.Vert( v.x, v.y, v.z )
		mesh.verts.append( bvert )
	for face in faces:
		print face.indices
		bface = Blender.NMesh.Face()
		bface.smooth = True
		bface.mode |= Blender.NMesh.FaceModes['TWOSIDE']
		bface.v.append( mesh.verts[face.indices[0]] )
		bface.v.append( mesh.verts[face.indices[1]] )
		bface.v.append( mesh.verts[face.indices[2]] )
		mesh.faces.append( bface )
	Blender.NMesh.PutRaw( mesh, "Teddy Object", True )
	#Creating a mirror image of the mesh
#	mirror_mesh = Blender.NMesh.GetRaw("Teddy Object")
#	for face in mirror_mesh.faces:
#		flipFace(face.v)
#	for vert in mirror_mesh.verts:
#		vert.co[2] *= -1
#	mirror_obj = Blender.NMesh.PutRaw(mirror_mesh, "Teddy Object Mirror")
	
def Run( ):
	init( ) #initialize the skeleton cal3d module
	#initialize skeleton data for blender
	b_armature = Blender.Object.New( 'Armature', 'arma' )
	b_armatureData = Blender.Armature.Armature( )
	b_armatureData.name = "Test"
	b_armatureData.autoIK = True
	#b_armatureData.drawAxes = True
	b_armatureData.envelopes = False
	b_armatureData.vertexGroups = True
	b_armatureData.drawType = Blender.Armature.OCTAHEDRON
	b_armature.link( b_armatureData )
	b_armatureData.makeEditable( )

	#make the bones
	fb_bone( skel, b_armatureData )

	b_armatureData.update( )
	b_scene = Blender.Scene.GetCurrent( )
	b_scene.link( b_armature )
	addMesh( )
		
