# --------------------------------------------------------------------------
# PyTeddy version .9 Apr 14, 2005
# Program versions: Blender 2.36+
# --------------------------------------------------------------------------
# ***** BEGIN GPL LICENSE BLOCK *****
#
# Copyright (C) 2005: Vamsi Krishna.V vkrishna _at_ research.iiit.ac.in, vkrishna _at_students.iiit.ac.in
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------

import math;

class Point3D:
	x = 0
	y = 0
	z = 0
	deleted = False 
	
	def __init__(self):
		pass

	#set the given values to point
	def setVal(self, x, y, z):
		self.x = x
		self.y = y
		self.z = z


class Line:
	"""Lines are of three types
	External edges and Internal Edges"""

	def __init__(self):
		self.p1 = Point3D()
		self.p2 = Point3D()
		self.setType( )
		
	def setType(self, type ="Internal"):
		self.type = type

	def getLen(self):
		xval = math.pow( (self.p1.x-self.p2.x ), 2 )
		yval = math.pow( (self.p1.y-self.p2.y ), 2 )
		return math.sqrt( xval*xval + yval*yval ) 

	def getMidPoint(self):
		tmp = Point3D( )
		tmp.setVal( (self.p1.x+self.p2.x)/2.0, (self.p1.y+self.p2.y)/2.0, 0 )
		return tmp

	def setPts(self, pt1, pt2):
		self.p1 = pt1
		self.p2 = pt2

class Triangle:
	"""Triangles can be of three types
	Terminal Triangles, Sleve Triangles, Junction Triangles"""
	#lines = []
	#points = []

	def __init__(self):
		self.type = "Terminal"
		self.lines = []
		self.indices = []
		self.processed = False 
		self.skeleton = [] 
		self.centroid = Point3D( )
	
	def setLines(self, lvec=[]):
		#self.points = []
		for i in xrange(len(lvec)):
			self.lines.append( lvec[i] )
		self.setType( )

	#SET THE VALUE OF THE MID POINT
	def setCentroid(self, pt):
		self.centroid.x = pt.x;
		self.centroid.y = pt.y;

	#ADDING SKELETON FROM PROGRAM
	def addSkeleton(self, skel):
		self.skeleton.append( skel )

	def setSkeleton(self):
		skel = []
		if self.type == "Junction":
			#JOIN THE MID POINTS OF LINES TO MEDIAN OF TRIANGLE
			for i in ( 0, 1, 2 ):
				lmid = self.lines[i].getMidPoint( )
				tmpLine = Line( )
				tmpLine.setPts( self.centroid, lmid )
				self.skeleton.append( tmpLine );
			#JOIN THE MID POINT OF LONGEST LINE TO MIDPOINTS
			#OF THE OTHER SIDES
#			lsideindex = 0
#			othmidpts = []
#			for i in ( 1, 2 ):
#				if self.lines[lsideindex].getLen( ) < self.lines[i].getLen( ):
#					lsideindex = i
#
#			for i in ( 0, 1, 2 ):
#				if i == lsideindex:
#					continue
#				othmidpts.append( self.lines[i].getMidPoint( ) )
#
#			lmidpt = self.lines[lsideindex].getMidPoint( )
#			tmpLine = Line( )
#			tmpLine.setPts( lmidpt, othmidpts[0] )
#			self.skeleton.append( tmpLine )
#			tmpLine = Line( )
#			tmpLine.setPts( lmidpt, othmidpts[1] )
#			self.skeleton.append( tmpLine )

		elif self.type == "Terminal":
			inidx = 0
			exidx  = 0
			for i in xrange( len( self.lines ) ):
				if self.lines[i].type == "Internal":
					inidx = i
				elif self.lines[i].type == "External":
					exidx = i
			secpt = Point3D( )
			externalline = self.lines[exidx]
			internalline = self.lines[inidx]
			if (externalline.p1.x == internalline.p1.x and externalline.p1.y == internalline.p1.y) or(externalline.p1.x == internalline.p2.x and externalline.p1.y == internalline.p2.y):
				secpt = externalline.p2;
			else:
				secpt = externalline.p1;
			lmidpt = self.lines[inidx].getMidPoint( )
			tmpLine = Line( )
			tmpLine.setPts( lmidpt, secpt )
			self.skeleton.append( tmpLine )

		elif self.type == "Sleeve":
			for line in self.lines:
				if line.type == "Internal":
					skel.append( line )
			if len(skel) == 2:
				tmpLine = Line( )
				tmpLine.setPts( skel[0].getMidPoint( ), skel[1].getMidPoint( ) )
				self.skeleton.append( tmpLine )
			else:
				print "Some thing Really Wrong dude"
		
	def setProcessed(self, val):
		self.processed = val

	def setIndices(self, ivec = ( ) ):
		for i in xrange( len(ivec) ):
			self.indices.append( ivec[i] )

	def setType(self):
		extedge = 0
		intedge = 0
		for line in self.lines:
			if line.type == "External":
				extedge += 1
			elif line.type == "Internal":
				intedge += 1
		if extedge == 2:
			self.type = "Terminal"
		elif extedge == 1:
			self.type = "Sleeve"
		else :
			self.type = "Junction"
